/* This is "physics.c", part of of the pool (billiards)-program
                   
                     "ANOTHER POOL" (V 0.97).

   "physics.c" includes the 'physic' of the game, such as the 
   collision of the balls, the reflection on the walls, ...

   Copyright (C) 1995 by Gerrit Jahn (email: ub1g@rz.uni-karlsruhe.de)

   "ANOTHER POOL" (V 0.97) is free software; you can redistribute it 
   and/or modify it under the terms of the GNU General Public License 
   as published by the Free Software Foundation; either version 2 of 
   the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU CC; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* ------------------------------ physics.c ------------------------------ */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <mousex.h>
#include "apool.h"

#define CURVES 2.5 /* wie stark bricht Weie bei 'Curve-Ball' senkr. aus */
#define CURVEP 7.5 /* wie stark bricht Weie bei 'Curve-Ball' parall. aus */

clock_t t1=0, old_time;
 
void calc_v_end( int n ) 
/* berechnet aus "eingegebener" Anfangsgeschwindigkeit der Kugel die 
   End-Roll- UND -Transl.geschwindigkeit; es ist v_tr = sqrt(5/7)*v_0.
   Es wird mit vex,y die (teilweise, eben zu sqrt(2/7) anteilige) Umwandlung
   von Translations- in Rotationsgeschwindigkeit beschrieben. Dadurch fngt
   die Kugel nach kurzem "Rutschen" an  zu "rollen" .... */
 {
/* double ges_energie, dummy;
 ges_energie = (dummy = k[n].v.x*k[n].v.x+k[n].v.y*k[n].v.y) + 
   sqrt(ROLL) * (k[n].vr.x*k[n].vr.x+k[n].vr.y*k[n].vr.y);
 if( dummy )
  {
  k[n].ve.x = sqrt(ROLL) * ges_energie * k[n].v.x/dummy;
  k[n].ve.y = sqrt(ROLL) * ges_energie * k[n].v.y/dummy;
  }
 if( k[n].vr.x || k[n].vr.y )
  {
  ??? !!! !!! !!!
  } */
 k[n].v.x += sqrt(ROLL)*k[n].vr.x;
 k[n].v.y += sqrt(ROLL)*k[n].vr.y;
 k[n].vr.x *= (1-sqrt(ROLL));
 k[n].vr.y *= (1-sqrt(ROLL));
 k[n].ve.x = sqrt(ROLL)*k[n].v.x;                                    
 k[n].ve.y = sqrt(ROLL)*k[n].v.y;
 }
 
void translate_spin( void ) 
/* Nachdem die Richtung der Geschw. feststeht, wird der Spin, der bisher im
   lokalen Koordinatensystem der Kugel festgelegt war auf globale (x,y)-
   Koordinaten umgerechnet. Das ist wichtig, da sich das lokale KS drehen
   kann, da auch "Curve-Balls" mglich sind... */
 {
 double puffer, dummy=0;  /* k.e.x : senkr., k.e.y: parall. zu k.v(x,y) !!! */
 struct vect s, add = { 0, 0 };
 /* senkr. Vektor zu v bestimmen: warum: s. unten */
 if( alph )
  {
  s.x = -k[WHITE].v.y; s.y = k[WHITE].v.x;
  if( (dummy = BETR( s )) )
   {
    s.x /= dummy; s.y /= dummy;
    dummy = sin( alph * M_PI / 180.0 );
    add.x = - s.x * dummy * k[WHITE].ez / DIFFX / CURVES;
    add.y = - s.y * dummy * k[WHITE].ez / DIFFX / CURVES;
   }
  s.x = k[WHITE].v.x; s.y = k[WHITE].v.y;
  if( (dummy = BETR( s )) ) /* dummy = dasselbe wie oben !!! !!! !!!*/
   {
   s.x /= dummy; s.y /= dummy;
   dummy = sin( alph * M_PI / 180.0 );
   add.x += s.x * dummy * k[WHITE].e.y / DIFFX / CURVEP;
   add.y += s.y * dummy * k[WHITE].e.y / DIFFX / CURVEP;
   }
  }
 /* bis hier wurde das "Ausbrechen" des weien Balls beim Curve-Ball,
    also die Zusatzgeschw., die der Ball durch den nicht mittigen Treffpkt
    bekommt, wenn der Queue noch dazu nicht parallel zum Tisch steht, 
    berechnet. jetzt kommt der "normale" Spin und Curve-Ball ... */
 if( (k[WHITE].v.x ) || (k[WHITE].v.y ) )
  {
  dummy = ( k[WHITE].ez*sin( M_PI*alph/180.0) ) / 
   sqrt(k[WHITE].v.x*k[WHITE].v.x + k[WHITE].v.y*k[WHITE].v.y);
  }
 puffer = k[WHITE].e.y;
 k[WHITE].e.y = k[WHITE].v.x * dummy; 
 k[WHITE].e.x = - k[WHITE].v.y * dummy;
 if( k[WHITE].v.x || k[WHITE].v.y )
  {
  dummy = puffer / 
   sqrt(k[WHITE].v.x*k[WHITE].v.x + k[WHITE].v.y*k[WHITE].v.y);
  }
 else dummy = 0;
 k[WHITE].e.y += (- k[WHITE].v.y * dummy); 
 k[WHITE].e.x += (- k[WHITE].v.x * dummy); 
 k[WHITE].ez *= cos( M_PI*alph/180.0);
 k[WHITE].e.x /= (RADIUS * DIFFX);
 k[WHITE].e.y /= (RADIUS * DIFFX);
 k[WHITE].v.x += add.x;
 k[WHITE].v.y += add.y;
}
 
void refl_kk( int n, struct vect p)   
/* Kollisionsberechnugn zw. Kugeln und der Knigsbande, die als infinitesimale
   Kugel an der jeweiligen Stelle angenommen wird, soda prinzipiell (fast)
   jeder Abprall-Winkel an diese Bande mglich ist, Die Berechnung erfolgt 
   hnlich wie der Sto zwischen zwei Kugeln (s. coll() ) und ansonsten analog
   reflextion() ... */
 {
 struct vect ab; /* ab := Abstandsvektor zw. den Mittelpunkten */
 double dummy;
 ab.x = (k[n].p.x*DIFFX-p.x); ab.y = (k[n].p.y*DIFFX-p.y); 
 ab.x /= (dummy = BETR( ab )); /* Abfrage, ob dummy != 0 ??? !!! !!! !!!*/
 ab.y /= dummy;
 if( (ab.x*k[n].v.x+ab.y*k[n].v.y) >= 0 ) return;
 /* Berechnung der Geschwindigkeit der Kugel nach der Kollision */       
 dummy = (k[n].v.x * ab.x + k[n].v.y * ab.y);
 k[n].v.x = (BANDEREF * (k[n].v.x - 2 * ab.x * dummy) 
  - BANDES * ab.y * dummy * k[n].ez);
 k[n].v.y = (BANDEREF * (k[n].v.y - 2 * ab.y * dummy)
  + BANDES * ab.x * dummy * k[n].ez);
 dummy = k[n].vr.x*ab.x + k[n].vr.y*ab.y;
 k[n].vr.x -= ab.x * dummy;
 k[n].vr.y -= ab.y * dummy;
 k[n].vr.x *= BANDEREF;
 k[n].vr.y *= BANDEREF;
 dummy = k[n].e.x * ab.x+k[n].e.y*ab.y;
 k[n].e.x -= ab.x * dummy;
 k[n].e.y -= ab.y * dummy;
 k[n].e.x *= BANDEREF;
 k[n].e.y *= BANDEREF;
 k[n].ez *= (1-BANDEREF);
 if( n == WHITE ) set_spin( k[WHITE].e.x, k[WHITE].e.y, k[WHITE].ez );
 calc_v_end( n );
 bande_hit = 1;
 }

void coll( int a, int b) 
/* Sto: Kollisionsberechnung zwischen den Kugeln a und b. Dabei wird die
   Masse der jeweiligen Kugel bercksichtigt, da die Weie wohl manchmal
   schwerer ist. Auerdem wird bercksichtigt, da Kugeln, die ROLLEN, nur
   ihre Translationsenergie abgeben knne, whrend sie die Rotationsenergie
   dazu befhigt, noch ein bichen weiter zu rollen (mit entsprechen 
   langsamerer Geschwindigkeit. Auerdem verlieren stoende Kugel einen 
   Teil ihres z-Spins, da ansonsten an den Banden interessante Effekt, wie
   negative Winkel auftreten knnen (immer noch ?? )*/
 { 
 struct vect ab, v; /* ab := Abstandsvektor zw. den Mittelpunkten */
 double dummy, dummy2;
 /* Abfrage, ob Kugeln kollidieren drfen. Das ist der Fall, wenn der
    Winkel zwischen Relativgeschw. und Abst.vektor < 90 ist */
 if( ( (ab.x = k[a].p.x-k[b].p.x) * (k[a].v.x-k[b].v.x) +
       (ab.y = k[a].p.y-k[b].p.y) * (k[a].v.y-k[b].v.y) ) >= 0 )
        return;
 /* Abstand i.allg. zu klein (wg. Zeitschritten) -> Korrektur: */
 v.x = k[a].v.x - k[b].v.x; v.y = k[a].v.y - k[b].v.y;
 dummy2 = SKALP( ab, v ) / SKALP( v, v );
 dummy = ( SKALP( ab, ab ) - 4*RADIUS*RADIUS/(DIFFX*DIFFX) )
  / SKALP( v, v );
 dummy2 += sqrt( dummy2 * dummy2 - dummy );
 k[a].p.x -= k[a].v.x * dummy2;
 k[a].p.y -= k[a].v.y * dummy2;
 k[b].p.x -= k[b].v.x * dummy2;
 k[b].p.y -= k[b].v.y * dummy2;
 ab.x = k[a].p.x - k[b].p.x;
 ab.y = k[a].p.y - k[b].p.y;
 /* Ende der "zu kleinen Abstands-korrektur" */
 dummy = BETR( ab );  /* ab = abst-vektor normieren */
 ab.x /= dummy;
 ab.y /= dummy;
 /* Abnahme des z-Spins, wenn Kugeln stoen (gibt's hoffentlich) */
 if( k[a].ez && (k[a].v.x || k[a].v.y) )
  k[a].ez *= (ab.x * k[a].v.x + ab.y*k[a].v.y) /  
   sqrt(k[a].v.x*k[a].v.x+k[a].v.y*k[a].v.y) * SPINC;
 if( k[b].ez && (k[b].v.x || k[b].v.y) )
  k[b].ez *= (ab.x * k[b].v.x + ab.y*k[b].v.y) / 
   sqrt(k[b].v.x*k[b].v.x+k[b].v.y*k[b].v.y) * SPINC;
 /* Berechnung der Geschwindigkeit der Kugeln nach der Kollision mittels: 
  v2'=(v1*a)*a+v2-(v2*a)*a; v1'=v1-(v1*a)*a+(v2*a)*a (|a|=1, a=Abst.vekt.) */
 dummy = (k[a].v.x - k[b].v.x) * ab.x + (k[a].v.y - k[b].v.y) * ab.y;
 k[a].v.x -= (2.0*k[b].m/(k[a].m+k[b].m)) * ab.x* dummy; /* mit untersch.  */
 k[a].v.y -= (2.0*k[b].m/(k[a].m+k[b].m)) * ab.y * dummy; /* MASSEN der Kug,*/
 k[b].v.x += (2.0*k[a].m/(k[a].m+k[b].m)) * ab.x * dummy;
 k[b].v.y += (2.0*k[a].m/(k[a].m+k[b].m)) * ab.y * dummy;
 /* Umrechnen der "briggebliebenen" Rot.-Geschw. in Rot.-, Transl.-Geschw. */
 dummy = sqrt(ROLL)*(ab.x*k[a].v.x+ab.y*k[a].v.y);
 k[a].v.x += ab.x*k[a].vr.x*dummy;  k[a].v.y += ab.y*k[a].vr.y*dummy;
 k[a].vr.x -= ab.x*k[a].vr.x*dummy; k[a].vr.y -= ab.y*k[a].vr.y*dummy;
 dummy = sqrt(ROLL)*(ab.x*k[b].v.x+ab.y*k[b].v.y);
 k[b].v.x += ab.x*k[b].vr.x*dummy;  k[b].v.y += ab.y*k[b].vr.y*dummy;
 k[b].vr.x -= ab.x*k[b].vr.x*dummy; k[b].vr.y -= ab.y*k[b].vr.y*dummy;
 /* neue "End-Geschwindigkeiten" der Kugeln berechnen */
 calc_v_end( a ); calc_v_end( b );
 k[a].p.x += k[a].v.x * dummy2;
 k[a].p.y += k[a].v.y * dummy2;
 k[b].p.x += k[b].v.x * dummy2;
 k[b].p.y += k[b].v.y * dummy2;
 if( !first_hit ) /* berprfen, ob Foul durch falsches Anspielen */
  { 
  if( a != WHITE ) first_hit = k[a].col; /* eigentlich albern, da WEISSE */
  else first_hit = k[b].col;               /* = letzte Kugel in Reihe...   */
  if( !freeball && ((ply[act].col && (first_hit != ply[act].col)) 
   || ((!ply[act].col) && (first_hit==COL_BLACK))) ) 
    ply[act].stat |= FOUL_WRONG_COLOR_TOUCHED;
  }
 if( anstoss ) anstoss = 0; /* fr Computer-Anstoss ... */
 }

void reflection( int n, int b ) 
/* Reflexion der Kugel n an der Bande b. Der Winkel, den die Bande hat, ist
   egal, es werden auch die schrgen Knigsbanden berechnet. An den Banden
   ndert sich die senkrecht zur Bande stehende Geschwindigkeitskomponente der
   Kugel in 2*die entgegengesetzte Richtung. Desweiteren wirkt sich der
   z-Spin aus. Ein Problem bei den Banden besteht darin, da Einfallswinkel
   NICHT gleich Ausfallswinkel ist, wodurch der Computer leichte Probleme
   bekommen knnte. Mal sehen, vielleicht "bescheit" er ... */
 { 
 double dummy;
 /* 1. Anteil senkrecht zur Bande, in Richtung des "Normalen-Vektors" ber. */
 dummy = ( k[n].v.x * ban[b].n.x + k[n].v.y * ban[b].n.y);
 /* Geschw. nach Reflexion, ein Teil normale Reflexion, ohne Spin, zum anderen
    die Umsetzung des z-Spins i.Abh. von dummy */
 k[n].v.x = (BANDEREF * (k[n].v.x-2*ban[b].n.x*dummy) 
  - BANDES * ban[b].n.y * dummy * k[n].ez);
 k[n].v.y = (BANDEREF * (k[n].v.y-2*ban[b].n.y*dummy)
  + BANDES * ban[b].n.x * dummy * k[n].ez); 
  /* Rotationsgeschwindigkeit senkrecht zur Bande verschwindet ! */
 dummy = k[n].vr.x*ban[b].n.x+k[n].vr.y*ban[b].n.y;
 k[n].vr.x -= ban[b].n.x * dummy;
 k[n].vr.y -= ban[b].n.y * dummy;
 k[n].vr.x *= BANDEREF;
 k[n].vr.y *= BANDEREF;
 /* Effet senkrecht zur Bande verschwindet ! */
 dummy = k[n].e.x * ban[b].n.x+k[n].e.y*ban[b].n.y;
 k[n].e.x -= ban[b].n.x * dummy;
 k[n].e.y -= ban[b].n.y * dummy;
 k[n].e.x *= BANDEREF;
 k[n].e.y *= BANDEREF;
 k[n].ez *= (1-BANDEREF);
 if( n == WHITE ) set_spin( k[WHITE].e.x, k[WHITE].e.y, k[WHITE].ez );
 calc_v_end( n );
 bande_hit = 1;
}

int in_line( int n, int b ) 
/* berprft, ob Mittelpt. der Kugel projeziert auf Banden-Gerade innerhalb 
   der beiden Grenzpunkte der Bande liegt und, ob der Abstand zwischen Bande
   und Kugel klein genug, ist, damit es zu eine Reflexion kommt ... */
 {
 struct vect kk, c;
 double t;
 c.x = ban[b].p0.x - ban[b].p1.x;
 c.y = ban[b].p0.y - ban[b].p1.y;
 t = - (c.x*(ban[b].p0.x-k[n].p.x*DIFFX) + c.y*(ban[b].p0.y-k[n].p.y*DIFFX)) /
       (c.x*c.x+c.y*c.y);
 kk.x = ban[b].p0.x + c.x * t;
 kk.y = ban[b].p0.y + c.y * t;
 return( (-c.x * (kk.x-ban[b].p0.x) >= 0) && (-c.y * (kk.y-ban[b].p0.y) >= 0)
  && (c.x * (kk.x-ban[b].p1.x) >= 0) && (c.y * (kk.y-ban[b].p1.y) >= 0) &&
  ( ((k[n].p.x*DIFFX-kk.x) * (k[n].p.x*DIFFX-kk.x) +
  (k[n].p.y*DIFFX-kk.y) * (k[n].p.y*DIFFX-kk.y)) <= (RADIUS)*(RADIUS) )); 
  /* oben: (RADIUS+0.5)^2 ??? !!! !!! !!! */
 }

void banden_test( void )  
 { /* Abfrage ob Kugeln Bande oder Lcher treffen ...  */ 
 int i, j;
 for(i=0;i<BALLS;i++)
  {
  /* Untenst. Abfrage kann man noch beschleunigen, indem man 1. die Abfrage
     bezgl. der groen Bande oben und unten in je zwei Teile links und rechts
     vom Mittelloch aufteilt und 2. die Knigsbanden nicht mehr abfragt,
     wenn bereits eine Reflexion an einer "normalen", groen Bande statt-
     gefunden hat... Aber auch so ist diese Prozedur schon umstndlich genug
     geschrieben ... (der Geschw. wegen) !!! !!! !!! */
  if( !k[i].stat && (k[i].v.x || k[i].v.y) )
   if( (k[i].p.x <= (RADIUS+1)/DIFFX) ) /* Kugel links in Nhe von Bande */
    {
    for( j=0;j<=2;j++ ) /* Abfrage, ob Kugel refl. an "groer" Bande */
     if( ((k[i].v.x*ban[j].n.x + k[i].v.y*ban[j].n.y) <= 0) && in_line(i, j) )
      { reflection( i, j ); break; }
    /* Reflexion an einer "Kugelbande" */
    if( DIFF( k[i].p.x-ban[1].p1.x/DIFFX, k[i].p.y-ban[1].p1.y/DIFFX ) <=
     RADIUS*RADIUS/(DIFFX*DIFFX) ) refl_kk(i,ban[1].p1); 
    else if( DIFF( k[i].p.x-ban[1].p0.x/DIFFX, k[i].p.y-ban[1].p0.y/DIFFX ) <=
     RADIUS*RADIUS/(DIFFX*DIFFX)) refl_kk(i,ban[1].p0);
    /* Kugel in Loch, das zu den entspr. Banden gehrt ? */
    for(j=0;j<=1;j++)
     if(DIFF(posl[j].p.x-k[i].p.x,posl[j].p.y-k[i].p.y) < 
      RADIUSL*RADIUSL/(DIFFX*DIFFX)) { delete_ball( i ); break; }
    }
   else if( k[i].p.x >= (DIFFX-RADIUS-1)/DIFFX ) /* Kugel rechts */
    {
     for( j=3;j<=5;j++ )
      if(((k[i].v.x*ban[j].n.x + k[i].v.y*ban[j].n.y) <= 0) && in_line( i, j))
       { reflection( i, j ); break; }
     if( DIFF( k[i].p.x-ban[4].p1.x/DIFFX, k[i].p.y-ban[4].p1.y/DIFFX ) <=
      RADIUS*RADIUS/(DIFFX*DIFFX) ) refl_kk(i,ban[4].p1); 
     else if( DIFF( k[i].p.x-ban[4].p0.x/DIFFX, k[i].p.y-ban[4].p0.y/DIFFX )
      <= RADIUS*RADIUS/(DIFFX*DIFFX)) refl_kk(i,ban[4].p0);
    for(j=3;j<=4;j++)
     if(DIFF(posl[j].p.x-k[i].p.x,posl[j].p.y-k[i].p.y)
      < RADIUSL*RADIUSL/(DIFFX*DIFFX)) { delete_ball( i ); break; }
    }
  if( !k[i].stat && (k[i].v.x || k[i].v.y) )
   if( (k[i].p.y <= (RADIUS+1)/DIFFX) ) /* Kugel oben */
    {
     for( j=6;j<=8;j++ ) /* geht was nicht !!! !!! !!! */
      { 
       if(((k[i].v.x*ban[j].n.x + k[i].v.y*ban[j].n.y)<=0) && in_line( i, j))
        { reflection( i, j ); break; }
       else if(((k[i].v.x*ban[j+6].n.x + k[i].v.y*ban[j+6].n.y)<=0) 
        && in_line(i, j+6)) { reflection( i, j+6 ); break; }
      }
     if( DIFF( k[i].p.x-ban[7].p1.x/DIFFX, k[i].p.y-ban[7].p1.y/DIFFX ) 
      <= RADIUS*RADIUS/(DIFFX*DIFFX) ) refl_kk(i,ban[7].p1); 
     else if( DIFF( k[i].p.x-ban[7].p0.x/DIFFX, k[i].p.y-ban[7].p0.y/DIFFX )
      <= RADIUS*RADIUS/(DIFFX*DIFFX)) refl_kk(i,ban[7].p0);
     else if( DIFF( k[i].p.x-ban[13].p1.x/DIFFX, k[i].p.y-ban[13].p1.y/DIFFX )
      <= RADIUS*RADIUS/(DIFFX*DIFFX) ) refl_kk(i,ban[13].p1); 
     else if( DIFF( k[i].p.x-ban[13].p0.x/DIFFX, k[i].p.y-ban[13].p0.y/DIFFX )
      <= RADIUS*RADIUS/(DIFFX*DIFFX)) refl_kk(i,ban[13].p0);
     for(j=1;j<=3;j++)
      if(DIFF(posl[j].p.x-k[i].p.x,posl[j].p.y-k[i].p.y) < 
       RADIUSL*RADIUSL/(DIFFX*DIFFX)) { delete_ball( i ); break; }
    }
   else if( k[i].p.y >= (DIFFX/2-RADIUS-1)/DIFFX ) /* Kugel unten */
    {
     for( j=9;j<=11;j++ )
      { 
       if(((k[i].v.x*ban[j].n.x + k[i].v.y*ban[j].n.y)<=0) && in_line( i, j)) 
        { reflection( i, j ); break; }
       else if(((k[i].v.x*ban[j+6].n.x + k[i].v.y*ban[j+6].n.y)<=0) 
        && in_line(i, j+6) ) { reflection( i, j+6 ); break; }
      }
     if( DIFF( k[i].p.x-ban[10].p1.x/DIFFX, k[i].p.y-ban[10].p1.y/DIFFX )
      <= RADIUS*RADIUS/(DIFFX*DIFFX) ) refl_kk(i,ban[10].p1); 
     else if( DIFF( k[i].p.x-ban[10].p0.x/DIFFX, k[i].p.y-ban[10].p0.y/DIFFX )
      <= RADIUS*RADIUS/(DIFFX*DIFFX)) refl_kk(i,ban[10].p0);
     else if( DIFF( k[i].p.x-ban[16].p1.x/DIFFX, k[i].p.y-ban[16].p1.y/DIFFX )
      <= RADIUS*RADIUS/(DIFFX*DIFFX) ) refl_kk(i,ban[16].p1); 
     else if( DIFF( k[i].p.x-ban[16].p0.x/DIFFX, k[i].p.y-ban[16].p0.y/DIFFX )
      <= RADIUS*RADIUS/(DIFFX*DIFFX)) refl_kk(i,ban[16].p0);
     for(j=4;j<=5;j++)
      if(DIFF(posl[j].p.x-k[i].p.x,posl[j].p.y-k[i].p.y) < 
       RADIUSL*RADIUSL/(DIFFX*DIFFX)) { delete_ball( i ); break; }
     if(DIFF(posl[0].p.x-k[i].p.x,posl[0].p.y-k[i].p.y) < 
      RADIUSL*RADIUSL/(DIFFX*DIFFX)) delete_ball( i );
    }
  }
 }

int move_balls( void ) 
/* Die Bewegung der Blle mit Reibung, Effet, Umwandlung von Translations-
   in Rotationsenergie, Umwandlung von Spin in Geschwindigkeit, etc.... */
 {
  int i, nomove = 0;
  double dummy;
  for(i=0;i<BALLS;i++)
   {
   if( !k[i].stat && (k[i].ve.x || k[i].ve.y || k[i].e.x || k[i].e.y) )
    {
     k[i].v.x -= ALPHA*( k[i].v.x - k[i].ve.x );  /* Umwandeln von Transl.- */
     k[i].v.y -= ALPHA*( k[i].v.y - k[i].ve.y );  /* in Rot.-Energie bzw.   */
     k[i].vr.x += ALPHA*( k[i].v.x - k[i].ve.x ); /* -Geschw.               */
     k[i].vr.y += ALPHA*( k[i].v.y - k[i].ve.y );
     if( (k[i].e.x || k[i].e.y) ) /* Umwandlung: Spin --> Geschw. */
      {
       k[i].v.x += sqrt(ROLL) * k[i].e.x * CETA * RADIUS;
       k[i].v.y += sqrt(ROLL) * k[i].e.y * CETA * RADIUS;
       k[i].vr.x += sqrt(1-ROLL) * k[i].e.x * CETA * RADIUS;
       k[i].vr.y += sqrt(1-ROLL) * k[i].e.y * CETA * RADIUS; 
       k[i].ve.x += (sqrt(1-ROLL) + sqrt(ROLL)) * k[i].e.x * CETA * RADIUS;
       k[i].ve.y += (sqrt(1-ROLL) + sqrt(ROLL)) * k[i].e.y * CETA * RADIUS;
       k[i].e.x *= (1-(sqrt(1-ROLL)+sqrt(ROLL)) * CETA) * (1-THETA);
       k[i].e.y *= (1-(sqrt(1-ROLL)+sqrt(ROLL)) * CETA) * (1-THETA);
       if( (k[i].e.x*k[i].e.x+k[i].e.y*k[i].e.y) < THETA*THETA/10000 ) 
        { k[i].e.x = k[i].e.y = 0; }
      }
     if( k[i].ve.x || k[i].ve.y ) /* Abfrage, ob Bewegungen noch "sichtbar" */
      {
       dummy = sqrt( k[i].ve.x*k[i].ve.x+k[i].ve.y*k[i].ve.y );
       k[i].ve.x -= k[i].ve.x/dummy * DELTA;
       k[i].ve.y -= k[i].ve.y/dummy * DELTA;
      }
     if( k[i].v.x || k[i].v.y ) /* Wenn man sich die Abfr. fr v, vr sparen */
      {				/* knnte, wre das Programm wohl schneller */
       dummy = sqrt(k[i].v.x*k[i].v.x+k[i].v.y*k[i].v.y);
       k[i].v.x  -= k[i].v.x/dummy * DELTA;
       k[i].v.y  -= k[i].v.y/dummy * DELTA;
      }
     if( k[i].vr.x || k[i].vr.y )
      {
       dummy = sqrt(k[i].vr.x*k[i].vr.x+k[i].vr.y*k[i].vr.y);
       k[i].vr.x -= k[i].vr.x/dummy * ETA;
       k[i].vr.y -= k[i].vr.y/dummy * ETA;
      }
     k[i].p.x += k[i].v.x;		/* Eigentliche Bewegung der Kugel */
     k[i].p.y += k[i].v.y;		/* unten: wann "ruht" die Kugel: */
     if( ( (k[i].ve.x*k[i].ve.x+k[i].ve.y*k[i].ve.y) < 10000*DELTA*DELTA)
      && !( k[i].e.x || k[i].e.y ) )
      { k[i].ve.x = k[i].ve.y = k[i].v.x = k[i].v.y = 0.0; }
     /* Abnahme vin .ez durch Bohrreibung ?? !!! !!! !!! */
     if( fabs(k[i].ez) > 31*BETA ) k[i].ez -= BETA * (2*(k[i].ez > 0)-1);
      else k[i].ez = 0.0;
    } 
   else nomove++;
   }
  if( (++counter > PAINT ) ) /* hier fehlt ein schneller Timer !!! !!! !!! */
   { /* wegen dem (t)ollen Grafiktreiber darf nicht jedesmal gemalt werden !*/
    if( BREMSE ) {while( !(rawclock() - old_time) ); old_time = rawclock();} 
    plot_balls();
    counter = 0; 
    fflush(datei);
   }            
 return nomove; /* Gibt an, wieviele Kugeln sich nicht mehr bewegen */
}

void stoss( void )   
/* Der eigentliche Sto, mit den Werten, die entweder der Spieler via Mouse
   oder der Computergegener hinsichtlich der Sto-Geschwindigkeit (Betrag
   und Richtung, Effet) liefert, wird die Weie losgelassen und die
   Kollisionen zwischen Kugeln und zw. Kugeln und Banden abgefragt... */
 {
  int i,j;
  save_actual_position(); 
  translate_spin();
  calc_v_end( WHITE );
  ply[act].speed = spd;
  old_time = rawclock();
  do
   {
    banden_test();
    for( i=0;i<BALLS;i++) /* Abfrage nach "potentieller" Kollision, d.h. */
     for(j=i;j<BALLS;j++) /* ob Abstand zw. Kugel i u. j klein genug !! */
      if( (j != i) && (!k[i].stat) && (!k[j].stat) &&
       (DIFF( k[i].p.x-k[j].p.x, k[i].p.y-k[j].p.y ) <= 
        4.0*RADIUS/DIFFX*RADIUS/DIFFX))
        coll( i, j ); 
   } 
  while( move_balls() < BALLS ); /* solange sich noch Blle bewegen */
  plot_balls();
 }
